from types.types import AccessToken, RefreshToken, BasicJWTToken
from utils.utils import get_now_timestamp
import jwt


def decode_access_token(token: str, secret: str) -> AccessToken:
    return AccessToken.model_validate(jwt.decode(token, secret, algorithms=["HS256"]))


def decode_refresh_token(token: str, secret: str) -> RefreshToken:
    return RefreshToken.model_validate(jwt.decode(token, secret, algorithms=["HS256"]))


def create_access_token(
    issuer: str, subject: str, duration_sec: int, roles: list[str], secret: str
) -> str:
    now_ts = get_now_timestamp()
    payload = AccessToken(
        iss=issuer,
        sub=subject,
        iat=now_ts,
        exp=now_ts + duration_sec * 1000,
        roles=roles,
    )
    return jwt.encode(payload.model_dump(), secret, algorithm="HS256")


def create_refresh_token(access_token: AccessToken, duration_sec: int, refresh_secret: str) -> str:
    now_ts = get_now_timestamp()
    payload = RefreshToken(
        iss=access_token.iss,
        sub=access_token.sub,
        iat=now_ts,
        exp=now_ts + duration_sec * 1000,
    )
    return jwt.encode(payload.model_dump(), refresh_secret, algorithm="HS256")


def is_token_valid(token: BasicJWTToken):
    now = get_now_timestamp()
    is_valid = now > token.iat and now < token.exp
    return is_valid


def token_has_role(token: AccessToken, role: str) -> bool:
    return role in token.roles
